@@ -15,65 +15,157 @@ import bs4
 
 import ChromeController
 
-@contextlib.contextmanager
-def _cr_context(cls):
-	with ChromeController.ChromeContext(cls._cr_binary) as cr:
-		cls._syncIntoChromium(cr)
-		yield cr
-		cls._syncOutOfChromium(cr)
-
 # Share the same chrome instance across multiple threads
 class ChromiumBorg(object):
-	__shared_state = {}
+	__shared_state = {
+		'_cr_binary' : None,
+	}
+
 	# init internal state variables here
 	__initialized = False
+	_cr_binary    = None
 
-	def _init_default_register(self, chrome_binary):
+	def _init_chromium_instance(self, chrome_binary):
 		# Current runners are configured to use 10 threads.
-		self.__cr = ChromeController.TabPooledChromium(binary=chrome_binary, tab_pool_max_size=10)
+		self.__cr = ChromeController.TabPooledChromium(binary=chrome_binary, tab_pool_max_size=10, *self._args, **self._kwargs)
 		self.__initialized = True
 
-	def __init__(self, chrome_binary):
+	def __init__(self, chrome_binary, *args, **kwargs):
 		self.__dict__ = self.__shared_state
+
+		if self._cr_binary != chrome_binary:
+			self._cr_binary = chrome_binary
+
+		self._args   = args
+		self._kwargs = kwargs
+
 		if not self.__initialized:
-			self._init_default_register(chrome_binary)
+			self._init_chromium_instance(chrome_binary)
 
 	def get(self):
+		if not self.__initialized:
+			self._init_chromium_instance()
 		return self.__cr
 
+	def close(self):
+		if self.__initialized:
+			self.__cr.close()
+			self.__initialized = False
+
+class ChromiumSingle(object):
+	def __init__(self, chrome_binary, *args, **kwargs):
+		self.__bin_name    = chrome_binary
+		self.__initialized = False
+		self.__cr          = None
+		self._args         = args
+		self._kwargs       = kwargs
+
+
+	def _init_chromium_instance(self):
+		# Current runners are configured to use 10 threads.
+		self.__cr = ChromeController.TabPooledChromium(binary=self.__bin_name, tab_pool_max_size=5, *self._args, **self._kwargs)
+		self.__initialized = True
+
+	def get(self):
+		if not self.__initialized:
+			self._init_chromium_instance()
+
+		return self.__cr
+
+	def close(self):
+		if self.__initialized:
+			self.__cr.close()
+			self.__initialized = False
 
 class WebGetCrMixin(object):
 	# creds is a list of 3-tuples that gets inserted into the password manager.
 	# it is structured [(top_level_url1, username1, password1), (top_level_url2, username2, password2)]
 	def __init__(self, use_global_tab_pool=True, *args, **kwargs):
-		if "chromium-binary" in kwargs:
-			self._cr_binary = kwargs.pop("chrome-binary")
+		'''
+		This adds a few chromium-related feature flags, and a management feature flag:
+
+		use_global_tab_pool - (default: True).
+			Specify whether multiple threads will share access to the same chrome instance.
+			Note that multiple threads can access multiple tabs at the same time without
+			issue. global tab management is serialized via locks.
+			This will (probably) fail in interesting ways if used in a multiprocessing context.
+		chromium_binary - (default: 'google-chrome')
+			Override the executed binary name. Defaults to `google-chrome` on $PATH.
+		chromium_headless - (default: True)
+			Should chromium be run in headless mode (e.g. `--headless` command line flag).
+			Running chrome with a head can be handy for debugging or dealing with particularly
+			obnoxious websites (since you can pause your code, interact with the browser, and
+			resume). This also allows plugins to operate (Plugins are not operative in headless
+			chrome).
+		chromium_enable_gpu - (default: False)
+			Should chromium be run without GPU support. This translates to the `--disable-gpu`
+			command line flag being passed if it is not set to True.
+			Primarily, this is for contexts where you may not have a GPU (VPSes, etc...).
+		'''
+		if "chromium_binary" in kwargs:
+			self._cr_binary = kwargs.pop("chromium_binary")
 		else:
 			self._cr_binary = "google-chrome"
+		if "chromium_headless" in kwargs:
+			self._cr_headless = kwargs.pop("chromium_headless")
+		else:
+			self._cr_headless = True
+
+		if "chromium_enable_gpu" in kwargs:
+			self._cr_enable_gpu = kwargs.pop("chromium_enable_gpu")
+		else:
+			self._cr_enable_gpu = False
 
 		super().__init__(*args, **kwargs)
 
-		self.navigate_timeout_secs = 10
-		self.wrapper_step_through_timeout = 20
+		self.navigate_timeout_secs = 15
+		self.wrapper_step_through_timeout = 60
+
+		self.use_global_tab_pool = use_global_tab_pool
 
 		if use_global_tab_pool:
-			self.borg_chrome_pool = True
-		else:
-			self.borg_chrome_pool = None
+			self.log.info("Using global chromium tab pool")
+
+		self.chrome_pool = None
+
+	def _chrome_context(self, itemUrl:str, extra_tid):
+		if not self.chrome_pool:
+			if self.use_global_tab_pool:
+				self.log.info("Initializing chromium pool on first use!")
+				self.chrome_pool = ChromiumBorg(
+						chrome_binary = self._cr_binary,
+						enable_gpu    = self._cr_enable_gpu,
+						headless      = self._cr_headless,
+					)
+			else:
+				self.chrome_pool = ChromiumSingle(
+						chrome_binary = self._cr_binary,
+						enable_gpu    = self._cr_enable_gpu,
+						headless      = self._cr_headless,
+					)
 
 
-	def _chrome_context(self, itemUrl, extra_tid):
-		if self.borg_chrome_pool and self.borg_chrome_pool is True:
-			self.log.info("Initializing chromium pool on first use!")
-			self.borg_chrome_pool = ChromiumBorg(chrome_binary=self._cr_binary)
 
-		if self.borg_chrome_pool:
-			assert itemUrl is not None, "You need to pass a URL to the contextmanager, so it can dispatch to the correct tab!"
-			return self.borg_chrome_pool.get().tab(url=itemUrl, extra_id=extra_tid)
-		else:
-			return ChromeController.ChromeContext(binary=self._cr_binary)
+		assert itemUrl is not None, "You need to pass a URL to the contextmanager, so it can dispatch to the correct tab!"
+		return self.chrome_pool.get().tab(url=itemUrl, extra_id=extra_tid)
+
+	@contextlib.contextmanager
+	def chrome_tab_context(self, url, extra_tid=None):
+		with self._chrome_context(url, extra_tid) as cr:
+
+			# If the tab has JS stopped, resume it
+			cr.Emulation_setScriptExecutionDisabled(False)
+
+			self._syncIntoChromium(cr)
+			yield cr
+			self._syncOutOfChromium(cr)
+
+			# Disable JS in the tab so it doesn't consume resources
+			cr.Emulation_setScriptExecutionDisabled(True)
+
 
 	def _syncIntoChromium(self, cr):
+		self.log.info("Syncing cookies into chromium")
 		cr.clear_cookies()
 		# Headers are a list of 2-tuples. We need a dict
 		hdict = dict(self.browserHeaders)
@@ -86,11 +178,14 @@ class WebGetCrMixin(object):
 				cr.set_cookie(cookie)
 
 	def _syncOutOfChromium(self, cr):
+		self.log.info("Syncing cookies out from chromium")
 		for cookie in cr.get_cookies():
 			self.cj.set_cookie(cookie)
 
 	def comprehensiveGetItemChromium(self, url, referrer=None, extra_tid=False, title_timeout=None, need_rendered=False):
 
+		assert url, "You have to pass a url!"
+
 		if title_timeout is None:
 			title_timeout = 1
 		if extra_tid is True:
@@ -99,8 +194,10 @@ class WebGetCrMixin(object):
 		ret = {}
 
 
-		with self._chrome_context(url, extra_tid=extra_tid) as cr:
+		with self._chrome_context(itemUrl=url, extra_tid=extra_tid) as cr:
 			# print("Starting nav (%s)" % need_rendered)
+
+			cr.Emulation_setScriptExecutionDisabled(False)
 			self._syncIntoChromium(cr)
 
 			################################################################################################
@@ -155,12 +252,16 @@ class WebGetCrMixin(object):
 				ret['resolved_content'] = cr.get_rendered_page_source()
 
 			self._syncOutOfChromium(cr)
+
+			cr.Emulation_setScriptExecutionDisabled(True)
 			# print("Done")
 
+		self.log.info("Chromium fetch complete.")
+
 		return ret
 
 
-	def getItemChromium(self, itemUrl, referrer=None, extra_tid=False, title_timeout=None):
+	def getItemChromium(self, itemUrl:str, referrer:str=None, extra_tid=False, title_timeout:int=None):
 		'''
 
 		'''
@@ -168,7 +269,7 @@ class WebGetCrMixin(object):
 		ret = self._unwaf_func("comprehensiveGetItemChromium", itemUrl, referrer=referrer, extra_tid=extra_tid, title_timeout=title_timeout)
 		return ret['raw_content'], ret['raw_filename'], ret['raw_mimetype']
 
-	def getHeadTitleChromium(self, url, referrer=None, extra_tid=False, title_timeout=None):
+	def getHeadTitleChromium(self, url:str, referrer:str=None, extra_tid=False, title_timeout:int=None):
 		'''
 
 		'''
@@ -184,7 +285,7 @@ class WebGetCrMixin(object):
 		}
 		return ret
 
-	def getHeadChromium(self, url, referrer=None, extra_tid=None):
+	def getHeadChromium(self, url:str, referrer:str=None, extra_tid=None):
 		'''
 
 		'''
@@ -195,7 +296,7 @@ class WebGetCrMixin(object):
 		return ret['resolved_url']
 
 
-	def chromiumGetRenderedItem(self, url, referrer=None, extra_tid=None, title_timeout=None):
+	def chromiumGetRenderedItem(self, url:str, referrer:str=None, extra_tid=None, title_timeout:int=None):
 		'''
 
 		'''
@@ -217,7 +318,7 @@ class WebGetCrMixin(object):
 		if hasattr(sup, '__del__'):
 			sup.__del__()
 
-	def stepThroughJsWaf_bare_chromium(self, url, titleContains='', titleNotContains='', extra_tid=None):
+	def stepThroughJsWaf_bare_chromium(self, url:str, titleContains:str='', titleNotContains:str='', extra_tid=None):
 		'''
 		Use Chromium to access a resource behind WAF protection.
 
@@ -247,6 +348,8 @@ class WebGetCrMixin(object):
 		if titleContains and titleNotContains:
 			raise ValueError("You can only pass a single conditional statement!")
 
+		assert url, "You have to pass a url!"
+
 		self.log.info("Attempting to access page through WAF browser verification.")
 
 		current_title = None
@@ -254,7 +357,8 @@ class WebGetCrMixin(object):
 		if extra_tid is True:
 			extra_tid = threading.get_ident()
 
-		with self._chrome_context(url, extra_tid=extra_tid) as cr:
+		with self._chrome_context(itemUrl=url, extra_tid=extra_tid) as cr:
+			cr.Emulation_setScriptExecutionDisabled(False)
 			self._syncIntoChromium(cr)
 			cr.blocking_navigate(url)
 
@@ -269,13 +373,14 @@ class WebGetCrMixin(object):
 					return True
 
 			self._syncOutOfChromium(cr)
+			cr.Emulation_setScriptExecutionDisabled(True)
 
 		self.log.error("Failed to step through. Current title: '%s'", current_title)
 
 		return False
 
 
-	def chromiumContext(self, url, extra_tid=None):
+	def chromiumContext(self, url:str, extra_tid=None):
 		'''
 		Return a active chromium context, useable for manual operations directly against
 		chromium.
@@ -290,4 +395,14 @@ class WebGetCrMixin(object):
 		if extra_tid is True:
 			extra_tid = threading.get_ident()
 
-		return self._chrome_context(url, extra_tid=extra_tid)
+		return self._chrome_context(itemUrl=url, extra_tid=extra_tid)
+
+	def close_chromium(self):
+		'''
+		If a chromium tab pool is open, close it.
+		Note that if you're using a shared tab pool, this will cause accesses to
+		other shared instances of the tab pool to potentially fail with exceptions..
+		'''
+		if self.chrome_pool:
+			self.chrome_pool.close()
+			self.chrome_pool = None
